# Copyright (c) 2010 Jonathan M. Lange. See LICENSE for details.

from testtools import TestCase
from testtools.matchers import Equals

from everyday.timeline import auto_tee, get_column_iterators, split_series


class TestAutoTee(TestCase):

    def test_singleton(self):
        data = [(1,)]
        (iter1,) = auto_tee(data)
        self.assertThat(list(iter1), Equals(data))

    def test_one_row_two_columns(self):
        data = [(1, 2)]
        iter1, iter2 = auto_tee(data)
        self.assertThat(list(iter1), Equals(data))
        self.assertThat(list(iter2), Equals(data))

    def test_one_row_three_columns(self):
        data = [(1, 2, 3)]
        iter1, iter2, iter3 = auto_tee(data)
        self.assertThat(list(iter1), Equals(data))
        self.assertThat(list(iter2), Equals(data))
        self.assertThat(list(iter3), Equals(data))

    def test_two_rows_one_column(self):
        data = [[1], [2]]
        (iter1,) = auto_tee(data)
        self.assertThat(list(iter1), Equals(data))

    def test_three_row_two_columns(self):
        data = [(1, 2), (3, 4), (5, 6)]
        iter1, iter2 = auto_tee(data)
        self.assertThat(list(iter1), Equals(data))
        self.assertThat(list(iter2), Equals(data))

    def test_many_columns_many_rows(self):
        data = [(1, 2, 'a'), (3, 4, 'b'), (5, 6, 'c')]
        iter1, iter2, iter3 = auto_tee(data)
        matcher = Equals(data)
        self.assertThat(list(iter1), matcher)
        self.assertThat(list(iter2), matcher)
        self.assertThat(list(iter3), matcher)

    def test_non_iterable(self):
        self.assertRaises(TypeError, auto_tee, 5)

    def test_iterable_no_columns(self):
        data = [1, 2, 3]
        iter1, = auto_tee(data)
        self.assertThat(list(iter1), Equals(data))

    def test_no_rows(self):
        data = []
        exception = self.assertRaises(ValueError, auto_tee, data)
        self.assertThat(
            str(exception), Equals("Cannot automatically tee empty sequence"))

    def test_one_row_no_columns(self):
        # If there's one row with no data in it, then we generate a list
        # containing no iterators.
        data = auto_tee([[]])
        self.assertThat(len(data), Equals(0))

    def test_many_rows_no_columns(self):
        # Likewise, even if there are many rows containing nothing, we cannot
        # generate one 'tee' per column, because there are no columns.
        data = auto_tee([[], [], []])
        self.assertThat(len(data), Equals(0))


class TestGetColumnIterators(TestCase):

    def test_two_columns_many_rows(self):
        data = [(1, 2), (3, 4), (5, 6)]
        odds, evens = get_column_iterators(data)
        self.assertThat(list(odds), Equals([1, 3, 5]))
        self.assertThat(list(evens), Equals([2, 4, 6]))


class TestSplitSeries(TestCase):

    def test_simple_case(self):
        # When we're given an iterator of 2-tuples, then split series should
        # return an iterator that yields exactly the same stuff as its input.
        data = [(1, 2), (3, 4), (5, 6)]
        [output] = split_series(data)
        self.assertThat(list(output), Equals(data))

    def test_many_y_values(self):
        data = [(1, 'a', 'z'), (2, 'b', 'y'), (3, 'c', 'x')]
        [one, two] = split_series(data)
        self.assertThat(list(one), Equals([(1, 'a'), (2, 'b'), (3, 'c')]))
        self.assertThat(list(two), Equals([(1, 'z'), (2, 'y'), (3, 'x')]))
