# Copyright (c) 2010 Jonathan M. Lange. See LICENSE for details.

"""Utilities for generating data for timeline graphs."""

__all__ = [
    'auto_tee',
    'csv2json',
    'date_to_timestamp',
    'series_dump',
    'series_dumps',
    'split_series',
    ]

import csv
import errno
import os
import shutil

from datetime import datetime
from itertools import count, izip, tee

import simplejson

from everyday.resources import get_html_path, load_html


def utc_to_timestamp(dt):
    """
    Python is so shit. Give me a datetime.totimestamp(), please.
    This only works with UTC timestamps.
    """
    dl = dt - datetime.utcfromtimestamp(0)
    return (dl.days * 60*60*24) + dl.seconds + (dl.microseconds / 1e6)


def iso_date_to_datetime(date_string):
    """Convert 'YYYY-MM-DD' to a datetime object."""
    return datetime.strptime(date_string, '%Y-%m-%d')


def date_to_timestamp(date_string):
    """Convert 'YYYY-MM-DD' to UNIX timestamp."""
    date = iso_date_to_datetime(date_string)
    return utc_to_timestamp(date)


def parse_csv_number(number_string):
    if number_string == '':
        return None
    return float(number_string)


def parse_csv_row(row):
    """Given a CSV row, return some parsed data."""
    date_string, numbers = row[0], row[1:]
    timestamp = date_to_timestamp(date_string) * 1000
    return [timestamp] + map(parse_csv_number, numbers)


def auto_tee(iterable):
    """Like 'tee', but guess 'n' based on the length of iterable's first value.
    """
    iterable = iter(iterable)
    try:
        first_row = iterable.next()
    except StopIteration:
        raise ValueError("Cannot automatically tee empty sequence")
    try:
        n = len(first_row)
    except TypeError:
        n = 1
    data = {}
    last_seen = [0]
    def gen(next):
        yield first_row
        for i in count():
             if i < last_seen[0]:
                  item = data[i].pop()
                  if not data[i]:
                       del data[i]
             else:
                  item = next()
                  data[i] = [item] * (n - 1)
                  last_seen[0] += 1
             yield item
    return [gen(iterable.next) for i in range(n)]


def iter_column(iterable, i):
    """
    Given an iterator of rows, iterate over a particular column in those rows.
    """
    return (row[i] for row in iterable)


def get_column_iterators(iterable):
    """
    Given an iterator that yields n-tuples, return n iterators that yield
    single values.
    """
    return [
        iter_column(teed, i) for (i, teed) in enumerate(auto_tee(iterable))]


def split_series(iterable):
    """
    Given an iterable of (x, y1, y2, ..., yn), return n iterables of (x, y1),
    (x, y2), ..., (x, yn).

    Useful for taking, say, a set of measurements over time and turning them
    into plottable series.
    """
    columns = get_column_iterators(iterable)
    x_axis, y_axes = columns[0], columns[1:]
    x_axes = tee(x_axis, len(y_axes))
    return [izip(x, y) for x, y in zip(x_axes, y_axes)]


def _evaluate(series):
    return [list(data) for data in series]


def series_dump(series, fd):
    simplejson.dump(_evaluate(series), fd)


def series_dumps(series):
    return simplejson.dumps(_evaluate(series))


def csv2json(csv_stream, has_header=False):
    csv_rows = csv.reader(csv_stream)
    output = {}
    if has_header:
        output['headers'] = csv_rows.next()[1:]
    output['data'] = _evaluate(
        split_series(parse_csv_row(row) for row in csv_rows))
    return simplejson.dumps(output)


def csv2flot_combined(title, csv_stream, has_header=False):
    json_data = csv2json(csv_stream, has_header)
    template = load_html('combined-template')
    return template % {'json_data': json_data, 'title': title}


def copy_if_not_exists(src, dest, recurse=False):
    if not os.path.exists(dest):
        if recurse:
            shutil.copytree(src, dest)
        else:
            shutil.copy(src, dest)


def csv2flot_split(csv_stream, output_directory, has_header=False, title=None):
    try:
        os.makedirs(output_directory)
    except OSError, e:
        if e.errno != errno.EEXIST:
            raise
    index = os.path.join(output_directory, 'index.html')
    if not os.path.exists(index):
        fd = open(index, 'w')
        template = load_html('split-template/index')
        fd.write(template % {'title': title})
        fd.close()
    copy_if_not_exists(
        get_html_path('split-template/plot.js'),
        os.path.join(output_directory, 'plot.js'))
    copy_if_not_exists(
        get_html_path('split-template/style.css'),
        os.path.join(output_directory, 'style.css'))
    copy_if_not_exists(
        get_html_path('flot'),
        os.path.join(output_directory, 'flot'),
        recurse=True)
    json_data = csv2json(csv_stream, has_header)
    fd = open(os.path.join(output_directory, 'data.json'), 'w')
    fd.write(json_data)
    fd.close()
